==> command-core-2.1.1-sources.jar: io/typst/command/Argument.java <==
package io.typst.command;

import io.typst.command.algebra.Tuple2;
import lombok.Data;
import lombok.With;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.function.Supplier;

@SuppressWarnings("DuplicatedCode")
@Data(staticConstructor = "of")
@With
public class Argument<A> {
    private final List<String> names;
    // args -> (result, remainingArgs)
    // List<String> -> (Option<A>, List<String>)
    private final Function<List<String>, Tuple2<Optional<A>, List<String>>> parser;
    // TODO: Function<List<String>, Supplier<Collection<String>>>
    private final List<Supplier<List<String>>> tabCompleters;

    public static <A> Argument<A> ofUnary(
            String name,
            Function<String, Optional<A>> parser,
            Supplier<List<String>> tabCompleter
    ) {
        return of(
                Collections.singletonList(name),
                args -> {
                    List<String> newArgs = new ArrayList<>(args);
                    String arg = newArgs.size() >= 1 ? newArgs.remove(0) : "";
                    return new Tuple2<>(
                            arg.length() >= 1 ? parser.apply(arg) : Optional.empty(),
                            newArgs
                    );
                },
                Collections.singletonList(tabCompleter)
        );
    }

    public Argument<Optional<A>> asOptional() {
        return new Argument<>(
                getNames(),
                args -> getParser().apply(args).map1(Optional::of),
                getTabCompleters()
        );
    }

    public <B> Argument<B> map(Function<A, B> f) {
        return new Argument<>(
                getNames(),
                args -> {
                    Tuple2<Optional<A>, List<String>> pair = getParser().apply(args);
                    return pair.map1(a -> a.map(f));
                },
                getTabCompleters()
        );
    }

    public Argument<A> withName(String name) {
        return withNames(Collections.singletonList(name));
    }

    public Argument<A> withTabCompleter(Supplier<List<String>> tabCompleter) {
        return withTabCompleters(Collections.singletonList(tabCompleter));
    }
}

==> command-core-3.0.0-sources.jar: io/typst/command/Argument.java <==
package io.typst.command;

import io.typst.command.algebra.Tuple2;
import lombok.Value;
import lombok.With;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.function.Supplier;

@SuppressWarnings("DuplicatedCode")
@Value(staticConstructor = "of")
@With
public class Argument<A> {
    String name;
    Class<?> classType;
    Function<List<String>, Tuple2<Optional<A>, List<String>>> parser;
    Supplier<List<String>> tabCompletes;

    public static <A> Argument<A> ofUnary(
            String name,
            Class<?> classType,
            Function<String, Optional<A>> parser,
            Supplier<List<String>> tabCompleter
    ) {
        return of(
                name,
                classType,
                args -> {
                    List<String> newArgs = new ArrayList<>(args);
                    String arg = newArgs.size() >= 1 ? newArgs.remove(0) : "";
                    return new Tuple2<>(
                            arg.length() >= 1 ? parser.apply(arg) : Optional.empty(),
                            newArgs
                    );
                },
                tabCompleter
        );
    }

    public Argument<Optional<A>> asOptional() {
        return new Argument<>(
                name,
                classType,
                args -> getParser().apply(args).map1(Optional::of),
                getTabCompletes()
        );
    }

    public <B> Argument<B> map(Function<A, B> f) {
        return new Argument<>(
                name,
                classType,
                args -> {
                    Tuple2<Optional<A>, List<String>> pair = getParser().apply(args);
                    return pair.map1(a -> a.map(f));
                },
                getTabCompletes()
        );
    }
}
